# the logging directories must be owned by the user that starts
# the circusd daemon, which is set in /etc/init/circus.conf
# we use a non-root user so that the celery workers start up correctly

[watcher:{{ app_name }}]
cmd = {{ app_home }}/bin/ceph-installer-gunicorn
working_dir={{ app_home }}/src/{{ app_name }}

stdout_stream.class = FileStream
stdout_stream.filename=/var/log/circus/{{ app_name }}-stdout.log

stderr_stream.class = FileStream
stderr_stream.filename=/var/log/circus/{{ app_name }}-stderr.log

[watcher:celery]
cmd = {{ app_home }}/bin/ceph-installer-celery
working_dir={{ app_home }}/src/{{ app_name }}/ceph_installer

stdout_stream.class = FileStream
stdout_stream.filename=/var/log/circus/celery-stdout.log

stderr_stream.class = FileStream
stderr_stream.filename=/var/log/circus/celery-stderr.log

[env:celery,{{ app_name }}]
PECAN_CONFIG = {{ app_home }}/src/{{ app_name }}/config/config.py
CEPH_ANSIBLE_PATH = {{ app_home }}/ceph-ansible
HOME = /home/{{ ansible_ssh_user }}
